@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 8 -*-
    rdesktop: A Remote Desktop Protocol client.
-   Copyright (C) Matthew Chapman 1999-2005
+   Copyright (C) Matthew Chapman 1999-2007
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -60,13 +60,16 @@ extern DEVICE_FNS serial_fns;
 extern DEVICE_FNS printer_fns;
 extern DEVICE_FNS parallel_fns;
 extern DEVICE_FNS disk_fns;
+#ifdef WITH_SCARD
+extern DEVICE_FNS scard_fns;
+#endif
 extern FILEINFO g_fileinfo[];
-extern BOOL g_notify_stamp;
+extern RD_BOOL g_notify_stamp;
 
 static VCHANNEL *rdpdr_channel;
 
 /* If select() times out, the request for the device with handle g_min_timeout_fd is aborted */
-NTHANDLE g_min_timeout_fd;
+RD_NTHANDLE g_min_timeout_fd;
 uint32 g_num_devices;
 
 /* Table with information about rdpdr devices */
@@ -91,7 +94,7 @@ struct async_iorequest *g_iorequest;
 
 /* Return device_id for a given handle */
 int
-get_device_index(NTHANDLE handle)
+get_device_index(RD_NTHANDLE handle)
 {
 	int i;
 	for (i = 0; i < RDPDR_MAX_DEVICES; i++)
@@ -114,7 +117,7 @@ convert_to_unix_filename(char *filename)
 	}
 }
 
-static BOOL
+static RD_BOOL
 rdpdr_handle_ok(int device, int handle)
 {
 	switch (g_rdpdr_device[device].device_type)
@@ -135,7 +138,7 @@ rdpdr_handle_ok(int device, int handle)
 }
 
 /* Add a new io request to the table containing pending io requests so it won't block rdesktop */
-static BOOL
+static RD_BOOL
 add_async_iorequest(uint32 device, uint32 file, uint32 id, uint32 major, uint32 length,
 		    DEVICE_FNS * fns, uint32 total_timeout, uint32 interval_timeout, uint8 * buffer,
 		    uint32 offset)
@@ -309,13 +312,16 @@ rdpdr_send_available(void)
 	channel_send(s, rdpdr_channel);
 }
 
-static void
+void
 rdpdr_send_completion(uint32 device, uint32 id, uint32 status, uint32 result, uint8 * buffer,
 		      uint32 length)
 {
 	uint8 magic[4] = "rDCI";
 	STREAM s;
 
+#ifdef WITH_SCARD
+	scard_lock(SCARD_LOCK_RDPDR);
+#endif
 	s = channel_init(rdpdr_channel, 20 + length);
 	out_uint8a(s, magic, 4);
 	out_uint32_le(s, device);
@@ -330,6 +336,9 @@ rdpdr_send_completion(uint32 device, uint32 id, uint32 status, uint32 result, ui
 	/* hexdump(s->channel_hdr + 8, s->end - s->channel_hdr - 8); */
 #endif
 	channel_send(s, rdpdr_channel);
+#ifdef WITH_SCARD
+	scard_unlock(SCARD_LOCK_RDPDR);
+#endif
 }
 
 static void
@@ -356,8 +365,8 @@ rdpdr_process_irp(STREAM s)
 	uint8 *buffer, *pst_buf;
 	struct stream out;
 	DEVICE_FNS *fns;
-	BOOL rw_blocking = True;
-	NTSTATUS status = STATUS_INVALID_DEVICE_REQUEST;
+	RD_BOOL rw_blocking = True;
+	RD_NTSTATUS status = RD_STATUS_INVALID_DEVICE_REQUEST;
 
 	in_uint32_le(s, device);
 	in_uint32_le(s, file);
@@ -395,6 +404,11 @@ rdpdr_process_irp(STREAM s)
 			break;
 
 		case DEVICE_TYPE_SCARD:
+#ifdef WITH_SCARD
+			fns = &scard_fns;
+			rw_blocking = False;
+			break;
+#endif
 		default:
 
 			error("IRP for bad device %ld\n", device);
@@ -415,7 +429,7 @@ rdpdr_process_irp(STREAM s)
 
 			if (length && (length / 2) < 256)
 			{
-				rdp_in_unistr(s, filename, length);
+				rdp_in_unistr(s, filename, sizeof(filename), length);
 				convert_to_unix_filename(filename);
 			}
 			else
@@ -425,7 +439,7 @@ rdpdr_process_irp(STREAM s)
 
 			if (!fns->create)
 			{
-				status = STATUS_NOT_SUPPORTED;
+				status = RD_STATUS_NOT_SUPPORTED;
 				break;
 			}
 
@@ -437,7 +451,7 @@ rdpdr_process_irp(STREAM s)
 		case IRP_MJ_CLOSE:
 			if (!fns->close)
 			{
-				status = STATUS_NOT_SUPPORTED;
+				status = RD_STATUS_NOT_SUPPORTED;
 				break;
 			}
 
@@ -448,7 +462,7 @@ rdpdr_process_irp(STREAM s)
 
 			if (!fns->read)
 			{
-				status = STATUS_NOT_SUPPORTED;
+				status = RD_STATUS_NOT_SUPPORTED;
 				break;
 			}
 
@@ -459,7 +473,7 @@ rdpdr_process_irp(STREAM s)
 #endif
 			if (!rdpdr_handle_ok(device, file))
 			{
-				status = STATUS_INVALID_HANDLE;
+				status = RD_STATUS_INVALID_HANDLE;
 				break;
 			}
 
@@ -468,7 +482,7 @@ rdpdr_process_irp(STREAM s)
 				buffer = (uint8 *) xrealloc((void *) buffer, length);
 				if (!buffer)
 				{
-					status = STATUS_CANCELLED;
+					status = RD_STATUS_CANCELLED;
 					break;
 				}
 				status = fns->read(file, buffer, length, offset, &result);
@@ -480,7 +494,7 @@ rdpdr_process_irp(STREAM s)
 			pst_buf = (uint8 *) xmalloc(length);
 			if (!pst_buf)
 			{
-				status = STATUS_CANCELLED;
+				status = RD_STATUS_CANCELLED;
 				break;
 			}
 			serial_get_timeout(file, length, &total_timeout, &interval_timeout);
@@ -488,11 +502,11 @@ rdpdr_process_irp(STREAM s)
 			    (device, file, id, major, length, fns, total_timeout, interval_timeout,
 			     pst_buf, offset))
 			{
-				status = STATUS_PENDING;
+				status = RD_STATUS_PENDING;
 				break;
 			}
 
-			status = STATUS_CANCELLED;
+			status = RD_STATUS_CANCELLED;
 			break;
 		case IRP_MJ_WRITE:
 
@@ -500,7 +514,7 @@ rdpdr_process_irp(STREAM s)
 
 			if (!fns->write)
 			{
-				status = STATUS_NOT_SUPPORTED;
+				status = RD_STATUS_NOT_SUPPORTED;
 				break;
 			}
 
@@ -512,7 +526,7 @@ rdpdr_process_irp(STREAM s)
 #endif
 			if (!rdpdr_handle_ok(device, file))
 			{
-				status = STATUS_INVALID_HANDLE;
+				status = RD_STATUS_INVALID_HANDLE;
 				break;
 			}
 
@@ -526,7 +540,7 @@ rdpdr_process_irp(STREAM s)
 			pst_buf = (uint8 *) xmalloc(length);
 			if (!pst_buf)
 			{
-				status = STATUS_CANCELLED;
+				status = RD_STATUS_CANCELLED;
 				break;
 			}
 
@@ -535,18 +549,18 @@ rdpdr_process_irp(STREAM s)
 			if (add_async_iorequest
 			    (device, file, id, major, length, fns, 0, 0, pst_buf, offset))
 			{
-				status = STATUS_PENDING;
+				status = RD_STATUS_PENDING;
 				break;
 			}
 
-			status = STATUS_CANCELLED;
+			status = RD_STATUS_CANCELLED;
 			break;
 
 		case IRP_MJ_QUERY_INFORMATION:
 
 			if (g_rdpdr_device[device].device_type != DEVICE_TYPE_DISK)
 			{
-				status = STATUS_INVALID_HANDLE;
+				status = RD_STATUS_INVALID_HANDLE;
 				break;
 			}
 			in_uint32_le(s, info_level);
@@ -562,7 +576,7 @@ rdpdr_process_irp(STREAM s)
 
 			if (g_rdpdr_device[device].device_type != DEVICE_TYPE_DISK)
 			{
-				status = STATUS_INVALID_HANDLE;
+				status = RD_STATUS_INVALID_HANDLE;
 				break;
 			}
 
@@ -578,7 +592,7 @@ rdpdr_process_irp(STREAM s)
 
 			if (g_rdpdr_device[device].device_type != DEVICE_TYPE_DISK)
 			{
-				status = STATUS_INVALID_HANDLE;
+				status = RD_STATUS_INVALID_HANDLE;
 				break;
 			}
 
@@ -594,7 +608,7 @@ rdpdr_process_irp(STREAM s)
 
 			if (g_rdpdr_device[device].device_type != DEVICE_TYPE_DISK)
 			{
-				status = STATUS_INVALID_HANDLE;
+				status = RD_STATUS_INVALID_HANDLE;
 				break;
 			}
 
@@ -608,7 +622,8 @@ rdpdr_process_irp(STREAM s)
 					in_uint8s(s, 0x17);
 					if (length && length < 2 * 255)
 					{
-						rdp_in_unistr(s, filename, length);
+						rdp_in_unistr(s, filename, sizeof(filename),
+							      length);
 						convert_to_unix_filename(filename);
 					}
 					else
@@ -631,19 +646,17 @@ rdpdr_process_irp(STREAM s)
 
 					in_uint32_le(s, info_level);	/* notify mask */
 
-					g_notify_stamp = True;
-
 					status = disk_create_notify(file, info_level);
 					result = 0;
 
-					if (status == STATUS_PENDING)
+					if (status == RD_STATUS_PENDING)
 						add_async_iorequest(device, file, id, major, length,
 								    fns, 0, 0, NULL, 0);
 					break;
 
 				default:
 
-					status = STATUS_INVALID_PARAMETER;
+					status = RD_STATUS_INVALID_PARAMETER;
 					/* JIF */
 					unimpl("IRP major=0x%x minor=0x%x\n", major, minor);
 			}
@@ -653,7 +666,7 @@ rdpdr_process_irp(STREAM s)
 
 			if (!fns->device_control)
 			{
-				status = STATUS_NOT_SUPPORTED;
+				status = RD_STATUS_NOT_SUPPORTED;
 				break;
 			}
 
@@ -665,25 +678,33 @@ rdpdr_process_irp(STREAM s)
 			buffer = (uint8 *) xrealloc((void *) buffer, bytes_out + 0x14);
 			if (!buffer)
 			{
-				status = STATUS_CANCELLED;
+				status = RD_STATUS_CANCELLED;
 				break;
 			}
 
 			out.data = out.p = buffer;
 			out.size = sizeof(buffer);
+
+#ifdef WITH_SCARD
+			scardSetInfo(device, id, bytes_out + 0x14);
+#endif
 			status = fns->device_control(file, request, s, &out);
 			result = buffer_len = out.p - out.data;
 
 			/* Serial SERIAL_WAIT_ON_MASK */
-			if (status == STATUS_PENDING)
+			if (status == RD_STATUS_PENDING)
 			{
 				if (add_async_iorequest
 				    (device, file, id, major, length, fns, 0, 0, NULL, 0))
 				{
-					status = STATUS_PENDING;
+					status = RD_STATUS_PENDING;
 					break;
 				}
 			}
+#ifdef WITH_SCARD
+			else if (status == (RD_STATUS_PENDING | 0xC0000000))
+				status = RD_STATUS_PENDING;
+#endif
 			break;
 
 
@@ -691,7 +712,7 @@ rdpdr_process_irp(STREAM s)
 
 			if (g_rdpdr_device[device].device_type != DEVICE_TYPE_DISK)
 			{
-				status = STATUS_INVALID_HANDLE;
+				status = RD_STATUS_INVALID_HANDLE;
 				break;
 			}
 
@@ -701,7 +722,7 @@ rdpdr_process_irp(STREAM s)
 			out.size = sizeof(buffer);
 			/* FIXME: Perhaps consider actually *do*
 			   something here :-) */
-			status = STATUS_SUCCESS;
+			status = RD_STATUS_SUCCESS;
 			result = buffer_len = out.p - out.data;
 			break;
 
@@ -710,7 +731,7 @@ rdpdr_process_irp(STREAM s)
 			break;
 	}
 
-	if (status != STATUS_PENDING)
+	if (status != RD_STATUS_PENDING)
 	{
 		rdpdr_send_completion(device, id, status, result, buffer, buffer_len);
 	}
@@ -817,7 +838,7 @@ rdpdr_process(STREAM s)
 	unimpl("RDPDR packet type %c%c%c%c\n", magic[0], magic[1], magic[2], magic[3]);
 }
 
-BOOL
+RD_BOOL
 rdpdr_init()
 {
 	if (g_num_devices > 0)
@@ -833,7 +854,7 @@ rdpdr_init()
 
 /* Add file descriptors of pending io request to select() */
 void
-rdpdr_add_fds(int *n, fd_set * rfds, fd_set * wfds, struct timeval *tv, BOOL * timeout)
+rdpdr_add_fds(int *n, fd_set * rfds, fd_set * wfds, struct timeval *tv, RD_BOOL * timeout)
 {
 	uint32 select_timeout = 0;	/* Timeout value to be used for select() (in millisecons). */
 	struct async_iorequest *iorq;
@@ -930,9 +951,9 @@ rdpdr_remove_iorequest(struct async_iorequest *prev, struct async_iorequest *ior
 
 /* Check if select() returned with one of the rdpdr file descriptors, and complete io if it did */
 static void
-_rdpdr_check_fds(fd_set * rfds, fd_set * wfds, BOOL timed_out)
+_rdpdr_check_fds(fd_set * rfds, fd_set * wfds, RD_BOOL timed_out)
 {
-	NTSTATUS status;
+	RD_NTSTATUS status;
 	uint32 result = 0;
 	DEVICE_FNS *fns;
 	struct async_iorequest *iorq;
@@ -961,7 +982,7 @@ _rdpdr_check_fds(fd_set * rfds, fd_set * wfds, BOOL timed_out)
 					/* iv_timeout between 2 chars, send partial_len */
 					/*printf("RDPDR: IVT total %u bytes read of %u\n", iorq->partial_len, iorq->length); */
 					rdpdr_send_completion(iorq->device,
-							      iorq->id, STATUS_SUCCESS,
+							      iorq->id, RD_STATUS_SUCCESS,
 							      iorq->partial_len,
 							      iorq->buffer, iorq->partial_len);
 					iorq = rdpdr_remove_iorequest(prev, iorq);
@@ -985,7 +1006,7 @@ _rdpdr_check_fds(fd_set * rfds, fd_set * wfds, BOOL timed_out)
 
 		}
 
-		rdpdr_abort_io(g_min_timeout_fd, 0, STATUS_TIMEOUT);
+		rdpdr_abort_io(g_min_timeout_fd, 0, RD_STATUS_TIMEOUT);
 		return;
 	}
 
@@ -1089,7 +1110,7 @@ _rdpdr_check_fds(fd_set * rfds, fd_set * wfds, BOOL timed_out)
 						out.size = sizeof(buffer);
 						out_uint32_le(&out, result);
 						result = buffer_len = out.p - out.data;
-						status = STATUS_SUCCESS;
+						status = RD_STATUS_SUCCESS;
 						rdpdr_send_completion(iorq->device, iorq->id,
 								      status, result, buffer,
 								      buffer_len);
@@ -1125,7 +1146,7 @@ _rdpdr_check_fds(fd_set * rfds, fd_set * wfds, BOOL timed_out)
 						{
 							g_notify_stamp = False;
 							status = disk_check_notify(iorq->fd);
-							if (status != STATUS_PENDING)
+							if (status != RD_STATUS_PENDING)
 							{
 								rdpdr_send_completion(iorq->device,
 										      iorq->id,
@@ -1151,7 +1172,7 @@ _rdpdr_check_fds(fd_set * rfds, fd_set * wfds, BOOL timed_out)
 }
 
 void
-rdpdr_check_fds(fd_set * rfds, fd_set * wfds, BOOL timed_out)
+rdpdr_check_fds(fd_set * rfds, fd_set * wfds, RD_BOOL timed_out)
 {
 	fd_set dummy;
 
@@ -1169,8 +1190,8 @@ rdpdr_check_fds(fd_set * rfds, fd_set * wfds, BOOL timed_out)
 
 
 /* Abort a pending io request for a given handle and major */
-BOOL
-rdpdr_abort_io(uint32 fd, uint32 major, NTSTATUS status)
+RD_BOOL
+rdpdr_abort_io(uint32 fd, uint32 major, RD_NTSTATUS status)
 {
 	uint32 result;
 	struct async_iorequest *iorq;
